import { ref, computed, isRef, onMounted, onUnmounted, watch, nextTick, openBlock, createElementBlock, normalizeClass, unref, normalizeStyle, createElementVNode, withDirectives, vShow, createCommentVNode, createBlock, Transition, withCtx, renderSlot } from "vue";
var style = "";
const MAIN_IMAGE_LOAD_SUCCESS = "success";
const MAIN_IMAGE_LOAD_ERROR = "error";
const IMAGE_POLL_INTERVAL = 10;
const IMAGE_BLUR = 20;
const IMAGE_RENDERING_DELAY = 0;
const IMAGE_ASPECT_RATIO = 0.5625;
const INTERSECTION_THRESHOLD = 0.2;
const useImage = (element) => {
  const image = new Image();
  const naturalWidth = ref(0);
  const naturalHeight = ref(0);
  const aspectRatio = computed(() => {
    return naturalWidth.value ? naturalHeight.value / naturalWidth.value : IMAGE_ASPECT_RATIO;
  });
  const pollImageData = setInterval(() => {
    if (image && image.naturalWidth) {
      clearInterval(pollImageData);
      naturalWidth.value = image.naturalWidth;
      naturalHeight.value = image.naturalHeight;
    }
  }, IMAGE_POLL_INTERVAL);
  const imageRenderer = (imageNode) => {
    const canvas = document.createElement("canvas");
    canvas.setAttribute("hidden", true);
    canvas.setAttribute("data-src", imageNode.src);
    canvas.width = 1;
    canvas.height = 1;
    document.body.appendChild(canvas);
    canvas.getContext("2d").drawImage(imageNode, 0, 0);
    document.body.removeChild(canvas);
  };
  const loadImage = () => {
    const imageNode = isRef(element) ? element.value : element;
    const src = imageNode.src;
    image.src = src;
    if (image.complete) {
      return Promise.resolve();
    }
    return new Promise((resolve, reject) => {
      image.onload = () => {
        imageRenderer(imageNode);
        resolve();
      };
      image.onerror = reject;
    });
  };
  return {
    image,
    aspectRatio,
    naturalWidth,
    naturalHeight,
    loadImage
  };
};
var useIntersect = (element) => {
  const isIntersected = ref(false);
  const options = {
    threshold: INTERSECTION_THRESHOLD
  };
  const observer = new IntersectionObserver((entries) => {
    if (entries[0].isIntersecting) {
      isIntersected.value = true;
      observer.disconnect();
    }
  }, options);
  const watchIntersectionOnce = (callback) => {
    const stop = watch(isIntersected, (is) => {
      if (is) {
        nextTick().then(callback);
        stop();
      }
    }, { immediate: true });
  };
  onMounted(() => {
    const el = isRef(element) ? element.value : element;
    observer.observe(el);
  });
  onUnmounted(() => {
    observer.disconnect();
  });
  return {
    watchIntersectionOnce,
    isIntersected
  };
};
const _hoisted_1 = ["src", "alt"];
const _hoisted_2 = ["src"];
const _hoisted_3 = {
  key: 2,
  class: "v-progressive-image-slot-default"
};
const _sfc_main = {
  props: {
    src: {
      type: String,
      required: true
    },
    placeholderSrc: {
      type: String
    },
    fallbackSrc: {
      type: String
    },
    delay: {
      type: [Number, String],
      default: IMAGE_RENDERING_DELAY
    },
    blur: {
      type: [Number, String],
      default: IMAGE_BLUR
    },
    alt: {
      type: String
    },
    customClass: {
      type: String
    },
    objectCover: {
      type: Boolean,
      default: false
    }
  },
  emits: [MAIN_IMAGE_LOAD_SUCCESS, MAIN_IMAGE_LOAD_ERROR],
  setup(__props, { emit }) {
    const props = __props;
    const rootRef = ref(null);
    const mainImageRef = ref(null);
    const isMainImageRendered = ref(false);
    const isFallbackImageRendered = ref(false);
    const isLoading = computed(() => !isMainImageRendered.value);
    const { isIntersected, watchIntersectionOnce } = useIntersect(rootRef);
    const { loadImage, aspectRatio, naturalWidth } = useImage(mainImageRef);
    const paddingHack = computed(() => ({
      paddingBottom: `${aspectRatio.value * 100}%`
    }));
    const componentStyle = computed(() => {
      if (props.objectCover) {
        return;
      }
      return {
        maxWidth: naturalWidth.value ? `${naturalWidth.value}px` : "100%"
      };
    });
    const placeholderStyle = computed(() => ({
      filter: `blur(${props.blur * 1}px)`
    }));
    const imageClasses = computed(() => {
      return [
        props.customClass,
        {
          "v-progressive-image-object-cover": props.objectCover
        }
      ];
    });
    const onComponentIntersected = () => {
      loadImage().then(() => {
        setTimeout(() => {
          isMainImageRendered.value = true;
          emit(MAIN_IMAGE_LOAD_SUCCESS);
        }, props.delay * 1);
      }).catch(() => {
        isMainImageRendered.value = true;
        isFallbackImageRendered.value = true;
        emit(MAIN_IMAGE_LOAD_ERROR, error);
      });
    };
    onMounted(() => {
      watchIntersectionOnce(onComponentIntersected);
    });
    return (_ctx, _cache) => {
      return openBlock(), createElementBlock("div", {
        ref_key: "rootRef",
        ref: rootRef,
        class: normalizeClass(["v-progressive-image", unref(imageClasses)]),
        style: normalizeStyle(unref(componentStyle))
      }, [
        createElementVNode("div", {
          style: normalizeStyle(unref(paddingHack))
        }, [
          unref(isIntersected) ? withDirectives((openBlock(), createElementBlock("img", {
            key: 0,
            class: "v-progressive-image-main",
            ref_key: "mainImageRef",
            ref: mainImageRef,
            src: isFallbackImageRendered.value ? __props.fallbackSrc : __props.src,
            alt: __props.alt
          }, null, 8, _hoisted_1)), [
            [vShow, isMainImageRendered.value]
          ]) : createCommentVNode("", true),
          __props.placeholderSrc ? (openBlock(), createBlock(Transition, {
            key: 1,
            appear: "",
            "leave-class": "v-progressive-image-fade-leave",
            "leave-to-class": "v-progressive-image-fade-leave-to",
            "leave-active-class": "v-progressive-image-fade-leave-active"
          }, {
            default: withCtx(() => [
              unref(isLoading) ? (openBlock(), createElementBlock("img", {
                key: 0,
                loading: "lazy",
                class: "v-progressive-image-placeholder",
                style: normalizeStyle(unref(placeholderStyle)),
                src: __props.placeholderSrc
              }, null, 12, _hoisted_2)) : createCommentVNode("", true)
            ]),
            _: 1
          })) : createCommentVNode("", true),
          _ctx.$slots.default ? (openBlock(), createElementBlock("div", _hoisted_3, [
            renderSlot(_ctx.$slots, "default", { isLoading: unref(isLoading) })
          ])) : createCommentVNode("", true)
        ], 4)
      ], 6);
    };
  }
};
const install = (app, props = {}) => {
  Object.keys(props).forEach((key) => {
    _sfc_main.props[key].default = props[key];
  });
  app.component("ProgressiveImage", _sfc_main);
};
export { _sfc_main as ProgressiveImage, install as default, install };
